@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -28,7 +28,8 @@
 #ifndef TOTEM_PL_PARSER_MINI
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h>
+
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
 #include "totem-pl-parser-pls.h"
@@ -44,18 +45,18 @@
 #define EXTVLCOPT "#EXTVLCOPT"
 
 static char *
-totem_pl_parser_url_to_dos (const char *url, const char *output)
+totem_pl_parser_url_to_dos (const char *url, GFile *output)
 {
 	char *retval, *i;
 
-	retval = totem_pl_parser_relative (url, output);
+	/* Get a relative URL if there is one */
+	retval = totem_pl_parser_relative (output, url);
 
 	if (retval == NULL)
 		retval = g_strdup (url);
 
 	/* Don't change URIs, but change smb:// */
-	if (g_str_has_prefix (retval, "smb://") != FALSE)
-	{
+	if (g_str_has_prefix (retval, "smb://") != FALSE) {
 		char *tmp;
 		tmp = g_strdup (retval + strlen ("smb:"));
 		g_free (retval);
@@ -78,64 +79,57 @@ totem_pl_parser_url_to_dos (const char *url, const char *output)
 
 gboolean
 totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
-		TotemPlParserIterFunc func, const char *output,
-		gboolean dos_compatible, gpointer user_data, GError **error)
+			   TotemPlParserIterFunc func, GFile *output,
+			   gboolean dos_compatible, gpointer user_data, GError **error)
 {
-	GnomeVFSHandle *handle;
-	GnomeVFSResult res;
+	GFileOutputStream *stream;
 	int num_entries_total, i;
 	gboolean success;
 	char *buf;
 	char *cr;
 
-	res = gnome_vfs_open (&handle, output, GNOME_VFS_OPEN_WRITE);
-	if (res == GNOME_VFS_ERROR_NOT_FOUND) {
-		res = gnome_vfs_create (&handle, output,
-				GNOME_VFS_OPEN_WRITE, FALSE,
-				GNOME_VFS_PERM_USER_WRITE
-				| GNOME_VFS_PERM_USER_READ
-				| GNOME_VFS_PERM_GROUP_READ);
-	}
-
-	if (res != GNOME_VFS_OK) {
-		g_set_error(error,
-			    TOTEM_PL_PARSER_ERROR,
-			    TOTEM_PL_PARSER_ERROR_VFS_OPEN,
-			    _("Couldn't open file '%s': %s"),
-			    output, gnome_vfs_result_to_string (res));
+	stream = g_file_replace (output, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
+	if (stream == NULL)
 		return FALSE;
-	}
 
 	cr = dos_compatible ? "\r\n" : "\n";
 	num_entries_total = gtk_tree_model_iter_n_children (model, NULL);
 	if (num_entries_total == 0)
 		return TRUE;
 
+	buf = g_strdup_printf ("#EXTM3U%s", cr);
+	success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
+	g_free (buf);
+	if (success == FALSE)
+		return FALSE;
+
 	for (i = 1; i <= num_entries_total; i++) {
 		GtkTreeIter iter;
 		char *url, *title, *path2;
 		gboolean custom_title;
+		GFile *file;
 
 		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
 			continue;
 
 		func (model, &iter, &url, &title, &custom_title, user_data);
 
-		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
-		{
+		file = g_file_new_for_uri (url);
+		if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
+			g_object_unref (file);
 			g_free (url);
 			g_free (title);
 			continue;
 		}
+		g_object_unref (file);
 
 		if (custom_title != FALSE) {
 			buf = g_strdup_printf (EXTINF",%s%s", title, cr);
-			success = totem_pl_parser_write_string (handle, buf, error);
+			success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 			g_free (buf);
 			if (success == FALSE) {
 				g_free (title);
 				g_free (url);
-				gnome_vfs_close (handle);
 				return FALSE;
 			}
 		}
@@ -143,7 +137,9 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 
 		if (dos_compatible == FALSE) {
 			char *tmp;
-			tmp = totem_pl_parser_relative (url, output);
+
+			tmp = totem_pl_parser_relative (output, url);
+
 			if (tmp == NULL && g_str_has_prefix (url, "file:")) {
 				path2 = g_filename_from_uri (url, NULL, NULL);
 			} else {
@@ -157,17 +153,14 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 		g_free (path2);
 		g_free (url);
 
-		success = totem_pl_parser_write_string (handle, buf, error);
+		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 		g_free (buf);
 
 		if (success == FALSE)
-		{
-			gnome_vfs_close (handle);
 			return FALSE;
-		}
 	}
 
-	gnome_vfs_close (handle);
+	g_object_unref (stream);
 
 	return TRUE;
 }
@@ -251,14 +244,15 @@ totem_pl_parser_parse_ram_url (TotemPlParser *parser, const char *url)
 }
 
 TotemPlParserResult
-totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
+totem_pl_parser_add_ram (TotemPlParser *parser, GFile *file, gpointer data)
 {
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents, **lines;
-	int size, i;
+	gsize size;
+	guint i;
 	const char *split_char;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	/* figure out whether we're a unix or dos RAM file */
@@ -280,14 +274,19 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 		/* Either it's a URI, or it has a proper path ... */
 		if (strstr(lines[i], "://") != NULL
 				|| lines[i][0] == G_DIR_SEPARATOR) {
+			GFile *line_file;
+
+			line_file = g_file_new_for_uri (lines[i]);
 			/* .ram files can contain .smil entries */
-			if (totem_pl_parser_parse_internal (parser, lines[i], NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+			if (totem_pl_parser_parse_internal (parser, line_file, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS)
 				totem_pl_parser_parse_ram_url (parser, lines[i]);
-			}
+			g_object_unref (line_file);
 		} else if (strcmp (lines[i], "--stop--") == 0) {
 			/* For Real Media playlists, handle the stop command */
 			break;
 		} else {
+			//FIXME
+#if 0
 			char *base;
 
 			/* Try with a base */
@@ -301,6 +300,7 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 				g_free (fullpath);
 			}
 			g_free (base);
+#endif
 		}
 	}
 
@@ -341,45 +341,27 @@ totem_pl_parser_get_extinfo_title (const char *extinfo)
 	return sep;
 }
 
-static char *
-totem_pl_parser_append_path (const char *base, const char *path)
-{
-	GnomeVFSURI *new, *baseuri;
-	char *fullpath;
-
-	baseuri = gnome_vfs_uri_new (base);
-	if (baseuri == NULL)
-		goto bail;
-	new = gnome_vfs_uri_append_path (baseuri, path);
-	gnome_vfs_uri_unref (baseuri);
-	if (new == NULL)
-		goto bail;
-	fullpath = gnome_vfs_uri_to_string (new, 0);
-	gnome_vfs_uri_unref (new);
-
-	return fullpath;
-
-bail:
-	return g_strdup_printf ("%s/%s", base, path);
-}
-
 TotemPlParserResult
-totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url,
-			const char *_base, gpointer data)
+totem_pl_parser_add_m3u (TotemPlParser *parser,
+			 GFile *file,
+			 GFile *base_file,
+			 gpointer data)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents, **lines;
-	int size, i;
+	gsize size;
+	guint i, num_lines;
+	gboolean dos;
 	const char *split_char, *extinfo;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	/* .pls files with a .m3u extension, the nasties */
 	if (g_str_has_prefix (contents, "[playlist]") != FALSE
 			|| g_str_has_prefix (contents, "[Playlist]") != FALSE
 			|| g_str_has_prefix (contents, "[PLAYLIST]") != FALSE) {
-		retval = totem_pl_parser_add_pls_with_contents (parser, url, _base, contents);
+		retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents);
 		g_free (contents);
 		return retval;
 	}
@@ -388,13 +370,19 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url,
 	extinfo = NULL;
 
 	/* figure out whether we're a unix m3u or dos m3u */
-	if (strstr(contents,"\x0d") == NULL)
+	if (strstr(contents,"\x0d") == NULL) {
 		split_char = "\n";
-	else
+		dos = FALSE;
+	} else {
 		split_char = "\x0d\n";
+		dos = TRUE;
+	}
 
 	lines = g_strsplit (contents, split_char, 0);
 	g_free (contents);
+	num_lines = g_strv_length (lines);
+	/* We don't count the terminating NULL */
+	num_lines--;
 
 	for (i = 0; lines[i] != NULL; i++) {
 		if (lines[i][0] == '\0')
@@ -409,26 +397,35 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url,
 			continue;
 		}
 
+		/* If we're on the last line, and we're a DOS M3U,
+		 * remove the terminating \n if it's there */
+		if (dos != FALSE && i == num_lines)
+			g_strchomp (lines[i]);
+
 		/* Either it's a URI, or it has a proper path ... */
 		if (strstr(lines[i], "://") != NULL
 				|| lines[i][0] == G_DIR_SEPARATOR) {
-			if (totem_pl_parser_parse_internal (parser, lines[i], NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+			GFile *url;
+
+			url = g_file_new_for_commandline_arg (lines[i]);
+			if (totem_pl_parser_parse_internal (parser, url, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 				totem_pl_parser_add_one_url (parser, lines[i],
 						totem_pl_parser_get_extinfo_title (extinfo));
 			}
+			g_object_unref (url);
 			extinfo = NULL;
 		} else if (g_ascii_isalpha (lines[i][0]) != FALSE
 			   && g_str_has_prefix (lines[i] + 1, ":\\")) {
 			/* Path relative to a drive on Windows, we need to use
 			 * the base that was passed to us */
-			char *fullpath;
+			GFile *url;
 
 			lines[i] = g_strdelimit (lines[i], "\\", '/');
 			/* + 2, skip drive letter */
-			fullpath = totem_pl_parser_append_path (_base, lines[i] + 2);
-			totem_pl_parser_add_one_url (parser, fullpath,
+			url = g_file_get_child (base_file, lines[i] + 2);
+			totem_pl_parser_add_one_file (parser, url,
 						     totem_pl_parser_get_extinfo_title (extinfo));
-			g_free (fullpath);
+			g_object_unref (url);
 			extinfo = NULL;
 		} else if (lines[i][0] == '\\' && lines[i][1] == '\\') {
 			/* ... Or it's in the windows smb form
@@ -447,17 +444,18 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url,
 			g_free (tmpurl);
 		} else {
 			/* Try with a base */
-			char *fullpath, *base, sep;
+			GFile *url, *_base_file;
+			char sep;
 
-			base = totem_pl_parser_base_url (url);
+			_base_file = g_file_get_parent (file);
 			sep = (split_char[0] == '\n' ? '/' : '\\');
 			if (sep == '\\')
 				lines[i] = g_strdelimit (lines[i], "\\", '/');
-			fullpath = totem_pl_parser_append_path (base, lines[i]);
-			totem_pl_parser_add_one_url (parser, fullpath,
+			url = g_file_get_child (_base_file, lines[i]);
+			g_object_unref (_base_file);
+			totem_pl_parser_add_one_file (parser, url,
 						     totem_pl_parser_get_extinfo_title (extinfo));
-			g_free (fullpath);
-			g_free (base);
+			g_object_unref (url);
 			extinfo = NULL;
 		}
 	}
@@ -468,18 +466,18 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url,
 }
 
 TotemPlParserResult
-totem_pl_parser_add_ra (TotemPlParser *parser, const char *url,
-			const char *base, gpointer data)
+totem_pl_parser_add_ra (TotemPlParser *parser,
+			GFile *file,
+			GFile *base_file, gpointer data)
 {
 	if (data == NULL || totem_pl_parser_is_uri_list (data, strlen (data)) == NULL) {
-		totem_pl_parser_add_one_url (parser, url, NULL);
+		totem_pl_parser_add_one_file (parser, file, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	return totem_pl_parser_add_ram (parser, url, NULL);
+	return totem_pl_parser_add_ram (parser, file, NULL);
 }
 
-
 #endif /* !TOTEM_PL_PARSER_MINI */
 
 #define CHECK_LEN if (i >= len) { return NULL; }
